<?php
/**
 * @copyright: Copyright © 2020 Firebear Studio. All rights reserved.
 * @author   : Firebear Studio <fbeardev@gmail.com>
 */

namespace Firebear\PlatformSap\Controller\Adminhtml\Order;

use Firebear\PlatformSap\Model\Order\UpdatePublisher;
use Magento\Backend\Model\View\Result\Redirect;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Sales\Model\OrderRepository;
use Psr\Log\LoggerInterface;
use Magento\Framework\Exception\LocalizedException;
use Firebear\PlatformSap\Model\Import\Source\Gateway\OrderStatus as OrderStatusGateway;

class Sync extends \Magento\Backend\App\Action
{
    const ADMIN_RESOURCE = 'Firebear_PlatformSap::index';

    /**
     * @var UpdatePublisher
     */
    private $orderUpdatePublisher;

    /**
     * @var OrderRepository
     */
    private $orderRepository;

    /**
     * @var LoggerInterface
     */
    private $logger;

    /**
     * @var OrderStatusGateway
     */
    private $orderStatusGateway;

    /**
     * @var ScopeConfigInterface
     */
    private $scopeConfig;

    /**
     * Sync constructor.
     * @param \Magento\Backend\App\Action\Context $context
     * @param UpdatePublisher $orderUpdatePublisher
     * @param OrderRepository $orderRepository
     * @param LoggerInterface $logger
     * @param OrderStatusGateway $orderStatusGateway
     * @param ScopeConfigInterface $scopeConfig
     */
    public function __construct(
        \Magento\Backend\App\Action\Context $context,
        UpdatePublisher $orderUpdatePublisher,
        OrderRepository $orderRepository,
        LoggerInterface $logger,
        OrderStatusGateway $orderStatusGateway,
        ScopeConfigInterface $scopeConfig
    ) {
        $this->orderUpdatePublisher = $orderUpdatePublisher;
        $this->orderRepository = $orderRepository;
        $this->logger = $logger;
        $this->orderStatusGateway = $orderStatusGateway;
        $this->scopeConfig = $scopeConfig;
        parent::__construct($context);
    }

    /**
     * @return Redirect|\Magento\Framework\App\ResponseInterface|\Magento\Framework\Controller\ResultInterface
     * @throws \Magento\Framework\Exception\InputException
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function execute()
    {
        $orderId = $this->getRequest()->getParam('order_id');
        $order = $this->orderRepository->get($orderId);

        /** @var Redirect $resultRedirect */
        $resultRedirect = $this->resultRedirectFactory->create();
        if (!$order->getId()) {
            $resultRedirect->setPath('sales/order/');
            return $resultRedirect;
        }

        try {
            $sapId = (int) $order->getSapId() > 0 ? (int) $order->getSapId() : '';
            if (!empty($sapId)) {
               $data = [
                    "sap_server_ip" => \trim(
                        $this->scopeConfig->getValue('firebear_importexport/sap/sap_server_ip')
                    ),
                    "sap_server_port" => \trim(
                        $this->scopeConfig->getValue('firebear_importexport/sap/sap_server_port')
                    ),
                    "sap_system_id" => \trim(
                        $this->scopeConfig->getValue('firebear_importexport/sap/sap_system_id')
                    ),
                    "sap_order_ipo" => \trim(
                        $this->scopeConfig->getValue('firebear_importexport/sap/sap_order_ipo')
                    ),
                   "sap_id" => $sapId,
                   "page" => 1
                ];
                $statusData = $this->orderStatusGateway->getSourceData($data);
                if (!empty($statusData)) {
                    $statusData = array_shift($statusData);
                    if (isset($statusData['CANCELED'])
                        && $statusData['CANCELED'] == 'N'
                        && isset($statusData['DocStatus'])
                        && $statusData['DocStatus'] !== 'C') {
                        $this->orderUpdatePublisher->execute($orderId);
                        $message = __('The order has been added to the queue for sync with SAP');
                        $this->messageManager->addSuccessMessage($message);
                    } elseif (isset($statusData['CANCELED']) && $statusData['CANCELED'] == 'Y') {
                        $message = __('The order is canceled in SBO. Unable to synchronize.');
                        $this->messageManager->addErrorMessage($message);
                    } elseif (isset($statusData['DocStatus']) && $statusData['DocStatus'] == 'C') {
                        $message = __('The order is closed in SBO. Unable to synchronize.');
                        $this->messageManager->addErrorMessage($message);
                    }
                } else {
                    $message = __('No content from API call');
                    $this->messageManager->addErrorMessage($message);
                }
            } else {
                $this->orderUpdatePublisher->execute($orderId);
                $message = __('The order has been added to the queue for sync with SAP');
                $this->messageManager->addSuccessMessage($message);
            }
            $resultRedirect->setPath('sales/order/view', ['order_id' => $orderId]);
        } catch (LocalizedException $e) {
            $this->logger->critical($e);
            $this->messageManager->addErrorMessage($e->getMessage());
            return $resultRedirect->setPath('sales/*');
        } catch (\Exception $e) {
            $this->logger->critical($e);
            $this->messageManager->addException($e, __('Error while processing order.'));
            return $resultRedirect->setPath('sales/*');
        }

        return $resultRedirect;
    }
}
