<?php
/**
 * @copyright: Copyright © 2021 Firebear Studio. All rights reserved.
 * @author   : Firebear Studio <fbeardev@gmail.com>
 */

namespace Firebear\PlatformSap\Model\Export\Adapter\Gateway;

use Firebear\ImportExport\Api\Export\History\CompressInterface;
use Firebear\ImportExport\Logger\Logger;
use Firebear\ImportExport\Model\Filesystem\File\ReadFactory;
use Firebear\ImportExport\Model\Source\Factory as SourceFactory;
use Firebear\ImportExport\Traits\General as GeneralTrait;
use Firebear\PlatformSap\Model\LogRepository;
use Magento\Framework\App\CacheInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Filesystem;
use Magento\Framework\Filesystem\Directory\WriteFactory as DirectoryWriteFactory;
use Magento\Framework\Filesystem\File\WriteFactory as FileWriteFactory;
use Magento\Framework\ObjectManagerInterface;
use Magento\Framework\Stdlib\DateTime\Timezone;
use Magento\Sales\Model\Order\InvoiceRepository;
use Symfony\Component\Console\Output\ConsoleOutput;
use Magento\Directory\Model\CountryFactory;
use Magento\Sales\Model\Order\Status\HistoryFactory;

/**
 * Sap customer gateway
 */
class Invoice extends SalesAbstractGateway
{
    use GeneralTrait;

    const SAP_ENTITY_ENDPOINT = 'ReserveInvoice/Create';

    /**
     * @var InvoiceRepository
     */
    protected $invoiceRepository;

    /**
     * @var ObjectManagerInterface
     */
    protected $objectManager;

    /**
     * @var CountryFactory
     */
    protected $countryFactory;

    /**
     * Invoice constructor.
     * @param ScopeConfigInterface $scopeConfig
     * @param InvoiceRepository $invoiceRepository
     * @param Logger $logger
     * @param ConsoleOutput $output
     * @param Filesystem $filesystem
     * @param ReadFactory $readFactory
     * @param DirectoryWriteFactory $writeFactory
     * @param FileWriteFactory $fileWrite
     * @param Timezone $timezone
     * @param SourceFactory $factory
     * @param CacheInterface $cache
     * @param LogRepository $logRepository
     * @param ObjectManagerInterface $objectManager
     * @param CompressInterface $compressCommand
     * @param CountryFactory $countryFactory
     * @throws \Magento\Framework\Exception\FileSystemException
     */
    public function __construct(
        ScopeConfigInterface $scopeConfig,
        InvoiceRepository $invoiceRepository,
        Logger $logger,
        ConsoleOutput $output,
        Filesystem $filesystem,
        ReadFactory $readFactory,
        DirectoryWriteFactory $writeFactory,
        FileWriteFactory $fileWrite,
        Timezone $timezone,
        SourceFactory $factory,
        CacheInterface $cache,
        LogRepository  $logRepository,
        ObjectManagerInterface $objectManager,
        CompressInterface $compressCommand,
        CountryFactory $countryFactory
    ) {
        parent::__construct(
            $scopeConfig,
            $filesystem,
            $readFactory,
            $writeFactory,
            $fileWrite,
            $timezone,
            $factory,
            $cache,
            $logRepository,
            $compressCommand,
            $countryFactory
        );
        $this->invoiceRepository = $invoiceRepository;
        $this->_logger = $logger;
        if (empty($this->_logger->getFileName())) {
            $this->_logger->setFileName('sapOrderUpdate');
        }
        $this->output = $output;
        $this->objectManager = $objectManager;
    }

    /**
     * @param $order
     * @param $invoice
     * @param array $data
     * @throws NoSuchEntityException
     */
    public function addInoivce($order, $invoice, $data = [])
    {
        $customerRegistry = $this->objectManager->create(\Magento\Customer\Model\CustomerRegistry::class);
        $addressRegistry = $this->objectManager->create(\Magento\Customer\Model\AddressRegistry::class);
        $taxCode = $this->scopeConfig->getValue('firebear_importexport/sap/sap_default_tax_code') ?? 'Exempt';
        $customerId = $order->getCustomerId();
        $sapId = (int) $invoice->getSapId() > 0 ? (int) $invoice->getSapId() : '';
        $sapEntityData = [
            'entity' => 'invoice',
            'entity_id' => $invoice->getEntityId(),
            'sap_id' => $sapId,
            'customer_id' => $customerId,
            'customer_name' => $order->getCustomerFirstname() . ' ' . $order->getCustomerLastname(),
            'email' => $order->getCustomerEmail(),
            'increment_id' => $invoice->getIncrementId(),
            'order_increment_id' => $order->getIncrementId(),
            'delivery_date' => date('Y-m-d', strtotime('+4 days')),
            'shipping_amount' => $invoice->getShippingAmount(),
            'ship_to' => '',
            'bill_to' => '',
            'status' => $order->getStatus(),
            'tax_code' => $taxCode
        ];

        if ($customerId) {
            $customer = $customerRegistry->retrieve($customerId);

            $sapId = $customer->getSapId();
            if (!empty($sapId)) {
                $sapEntityData['customer_sap_id'] = $sapId;
            }
        }

        if ($order->getShippingAddress()->getCustomerAddressId()) {
            $address = false;
            $addressSapId = false;
            try {
                $address = $addressRegistry->retrieve($order->getShippingAddress()->getCustomerAddressId())->getDataModel();
                $addressSapId = $address->getCustomAttribute('sap_id');
            } catch (\Exception $e) {
            }

            if (!empty($addressSapId)) {
                $sapEntityData['ship_to'] = $addressSapId->getValue();
            } elseif ($address) {
                $sapEntityData['ship_to'] = 'Ship To #' . $order->getShippingAddress()->getCustomerAddressId();
            } else {
                $sapEntityData['ship_to'] = '';
            }
        }
        if ($order->getBillingAddress()->getCustomerAddressId()) {
            $address = false;
            $addressSapId = false;
            try {
                $address = $addressRegistry->retrieve($order->getBillingAddress()->getCustomerAddressId())->getDataModel();
                $addressSapId = $address->getCustomAttribute('sap_id');
            } catch (\Exception $e) {
            }

            if (!empty($addressSapId)) {
                $sapEntityData['bill_to'] = $addressSapId->getValue();
            } elseif ($address) {
                $sapEntityData['bill_to'] = 'Bill To #' . $order->getBillingAddress()->getCustomerAddressId();
            } else {
                $sapEntityData['bill_to'] = '';
            }
        }

        $lineNum = 0;
        foreach ($invoice->getAllItems() as $item) {
            $sapItem = [
                'line_num' => $lineNum++,
                'sku' => $item->getSku(),
                'product_name' => $item->getName(),
                'qty' => floatval($item->getQty()),
                'price' => floatval($item->getPrice()),
                'discount_percent' => floatval($item->getDiscountPercent()),
                'tax_code' => $taxCode,
            ];
            if ($item->getDiscountPercent() == 0 && $item->getDiscountAmount() > 0) {
                $discountPercent = $item->getDiscountAmount() * 100 / $item->getPrice();
                $sapItem['discount_percent'] = $discountPercent;
            }
            $sapEntityData['items'][] = $sapItem;
        }

        $sapEntityData['addresses'] = $this->getOrderAddresses($order);
        $sapEntityData['shipping_address_text'] = $sapEntityData['addresses']['shipping_address_text'];
        $sapEntityData['billing_address_text'] = $sapEntityData['addresses']['billing_address_text'];
        unset($sapEntityData['addresses']['shipping_address_text']);
        unset($sapEntityData['addresses']['billing_address_text']);

        $updateResponse = $this->uploadSourceData($sapEntityData);

        if (!$updateResponse['error']) {
            if (isset($updateResponse['object_id'])) {
                $invoice->setData('sap_id', $updateResponse['object_id']);
                $this->invoiceRepository->save($invoice);
            }

            $this->addLogWriteln($updateResponse['message'], $this->output);
        } else {
            $this->addLogWriteln($updateResponse['message'], $this->output, 'error');
        }
    }
}
