<?php
/**
 * Copyright © Firebear Studio, Inc. All rights reserved.
 * See COPYING.txt for license details.
 */
namespace Firebear\PlatformSap\Model\Import\Source;

use Firebear\PlatformSap\Model\Import\Source\Gateway\CatalogRule as CatalogPriceGateway;
use Magento\Catalog\Model\ResourceModel\Category\CollectionFactory;
use Magento\CatalogRule\Model\CatalogRuleRepository;
use Magento\CatalogRule\Model\Rule\Condition\Combine;
use Magento\CatalogRule\Model\Rule\Condition\Product;
use Magento\Customer\Api\GroupRepositoryInterface;
use Magento\Framework\Api\SearchCriteriaBuilder;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\Serialize\Serializer\Json;
use Magento\Store\Model\StoreManagerInterface;

/**
 * Sap catalog price source
 */
class CatalogRule extends AbstractSource
{
    const DEFAULT_WEBSITE_IDS = 1;

    /**
     * Column category name.
     */
    const CATEGORY_COL_NAME = 'name';

    /**
     * Delimiter in category path.
     */
    const DELIMITER_CATEGORY = '/';

    const GROUP_SORT_ORDER = 2;

    const MANUFACTURER_SORT_ORDER = 4;

    const DEFAULT_SORT_ORDER = 6;

    /**
     * @var CatalogPriceGateway
     */
    protected $gateway;

    /**
     * Config data
     *
     * @var array
     */
    protected $_data = [];

    /**
     * @var RequestInterface
     */
    protected $request;

    /**
     * @var Json
     */
    protected $jsonSerializer;

    /**
     * Categories text-path to ID hash.
     *
     * @var array
     */
    protected $categories = [];

    /**
     * @var StoreManagerInterface
     */
    protected $storeManager;

    /**
     * @var CollectionFactory
     */
    protected $categoryColFactory;

    /**
     * @var GroupRepositoryInterface
     */
    protected $groupRepository;

    /**
     * @var SearchCriteriaBuilder
     */
    protected $searchCriteriaBuilder;

    /**
     * @var array
     */
    protected $customerGroups;

    /**
     * @var array
     */
    protected $customerGroupsBySapId;

    /**
     * @var \Magento\CatalogRule\Model\ResourceModel\Rule\CollectionFactory
     */
    protected $catalogRuleColFactory;

    /**
     * @var array
     */
    protected $catalogRules;

    /**
     * @var CatalogRuleRepository
     */
    protected $catalogRuleRepository;

    /**
     * CatalogRule constructor.
     *
     * @param CatalogPriceGateway                                             $gateway
     * @param RequestInterface                                                $request
     * @param Json                                                            $jsonSerializer
     * @param StoreManagerInterface                                           $storeManager
     * @param CollectionFactory                                               $categoryColFactory
     * @param GroupRepositoryInterface                                        $groupRepository
     * @param SearchCriteriaBuilder                                           $searchCriteriaBuilder
     * @param \Magento\CatalogRule\Model\ResourceModel\Rule\CollectionFactory $catalogRuleColFactory
     * @param CatalogRuleRepository                                           $catalogRuleRepository
     * @param null                                                            $data
     *
     * @throws \Exception
     */
    public function __construct(
        CatalogPriceGateway $gateway,
        RequestInterface $request,
        Json $jsonSerializer,
        StoreManagerInterface $storeManager,
        CollectionFactory $categoryColFactory,
        GroupRepositoryInterface $groupRepository,
        SearchCriteriaBuilder $searchCriteriaBuilder,
        \Magento\CatalogRule\Model\ResourceModel\Rule\CollectionFactory $catalogRuleColFactory,
        CatalogRuleRepository $catalogRuleRepository,
        $data = null
    ) {
        $this->gateway = $gateway;
        $this->request = $request;
        $this->jsonSerializer = $jsonSerializer;
        $this->storeManager = $storeManager;
        $this->categoryColFactory = $categoryColFactory;
        $this->groupRepository = $groupRepository;
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
        $this->catalogRuleColFactory = $catalogRuleColFactory;
        $this->catalogRuleRepository = $catalogRuleRepository;
        $this->parseEntities($data);
    }

    /**
     * @param $data
     * @return array|bool
     * @throws \Exception
     */
    protected function parseEntities($data)
    {
        $catalogRules = $this->getCatalogRules();
        $this->entities = parent::parseEntities($data);
        if (!empty($this->entities)) {
            foreach ($this->entities as $key => $entity) {
                $entityConditions = $this->prepareConditions($entity);
                $ruleName = $this->getRuleName($entity);

                if (!empty($entityConditions)) {
                    $conditions = [
                        'type' => Combine::class,
                        'attribute' => 0,
                        'operator' => 0,
                        'value' => 1,
                        'is_value_processed' => 0,
                        'aggregator' => "all",
                        'conditions' => $entityConditions
                    ];

                    $this->initCustomerGroups();
                    if ($this->validateData($entity, 'CustomerGroup')
                        && isset($this->customerGroups[$entity['CustomerGroup']])) {
                        $customerGroupIds = $this->customerGroups[$entity['CustomerGroup']];
                    } elseif ($this->validateData($entity, 'CustomerGroup')
                        && !isset($this->customerGroups[$entity['CustomerGroup']])) {
                        continue;
                    } else {
                        $customerGroupIds = $this->getAllCustomerGroups();
                    }

                    $isActive = 1;
                    if (!empty($this->entities[$key]['ValidForm'])) {
                        $now = new \DateTime();
                        $fromDate = new \DateTime($this->entities[$key]['ValidForm']);
                        $isActive = ($now > $fromDate);
                    }
                    if (!empty($this->entities[$key]['ValidTo'])) {
                        $now = new \DateTime();
                        $toDate = new \DateTime($this->entities[$key]['ValidTo']);
                        $isActive = ($now < $toDate);
                    }

                    $sapInternalId = $entity['AbsEntry'] . '-' . $entity['ObjType'] . '-' . $entity['ObjKey'];

                    $this->entities[$key] = [
                        'name' => $ruleName,
                        'description' => '',
                        'website_ids' => self::DEFAULT_WEBSITE_IDS,
                        'customer_group_ids' => $customerGroupIds,
                        'from_date' => ($this->validateData($entity, 'ValidForm')) ? $entity['ValidForm'] : '',
                        'to_date' => ($this->validateData($entity, 'ValidForm')) ? $entity['ValidTo'] : '',
                        'conditions_serialized' => $this->jsonSerializer->serialize($conditions),
                        'stop_rules_processing' => 1,
                        'simple_action' => 'by_percent',
                        'discount_amount' => $entity['Discount'],
                        'sort_order' => $this->getSortOrder($this->entities[$key]),
                        'is_active' => $isActive,
                        'sap_id' => $sapInternalId
                    ];

                    if (isset($catalogRules[$sapInternalId])) {
                        $this->entities[$key]['rule_id'] = $catalogRules[$sapInternalId];
                    }
                } else {
                    unset($this->entities[$key]);
                }
            }
        }

        if (!empty($this->entities)) {
            $this->deleteExistingRules();
        }
        $this->_colNames = array_keys($this->entities[0] ?? []);
        return $this->entities;
    }

    /**
     * Delete unused rules
     *
     * @return $this
     * @throws \Magento\Framework\Exception\CouldNotDeleteException
     */
    protected function deleteExistingRules()
    {
        $existingRules = $this->getCatalogRules();
        $rulesToImport = [];
        if (!empty($this->entities)) {
            foreach ($this->entities as $rule) {
                $rulesToImport[$rule['sap_id']] = 1;
            }
        }

        $rulesToDelete = array_diff_key($existingRules, $rulesToImport);

        foreach ($rulesToDelete as $rule) {
            if ($rule > 0) {
                $this->catalogRuleRepository->deleteById($rule);
            }
        }

        return $this;
    }

    /**
     * Get rule priority.
     * SAP Priority: Item Groups -> Manufacturers -> Items
     *
     * @param array $entity
     *
     * @return int
     */
    protected function getSortOrder(array $entity)
    {
        if ($this->validateData($entity, 'ItemGroup')) {
            return self::GROUP_SORT_ORDER;
        }
        if ($this->validateData($entity, 'Manufacturer')) {
            return self::MANUFACTURER_SORT_ORDER;
        }

        return self::DEFAULT_SORT_ORDER;
    }

    protected function getRuleName(array $entity)
    {
        $name = 'SAP';

        if (!empty($entity['Type'])) {
            if ($entity['Type'] == 'C' && $this->validateData($entity, 'CustomerGroup')) {
                $name .= ' - Customer Group: ' . $entity['CustomerGroup'];
            } else {
                $name .= ' - All Customer Groups';
            }
        }
        if ($this->validateData($entity, 'ItemGroup')) {
            $name .= ' - Category: ' . $entity['ItemGroup'];
        }
        if ($this->validateData($entity, 'Manufacturer')) {
            $name .= ' - Manufacturer: ' . $entity['Manufacturer'];
        }
        if ($this->validateData($entity, 'ItemCode')) {
            $name .= ' - Product: ' . $entity['ItemCode'];
        }

        return $name;
    }

    /**
     *  Init Magento catalog rules
     */
    protected function getCatalogRules()
    {
        if (empty($this->catalogRules)) {
            $this->catalogRules = [];
            $catalogRules = $this->catalogRuleColFactory->create();
            foreach ($catalogRules as $catalogRule) {
                $sapInternalId = $catalogRule->getData('sap_id');
                if (!empty($sapInternalId)) {
                    $this->catalogRules[$sapInternalId] = $catalogRule->getId();
                }
            }
        }
        return $this->catalogRules;
    }

    /**
     * @param $entity
     * @return array
     */
    protected function prepareConditions($entity)
    {
        $this->initCategories();
        $conditions = [];
        if ($this->validateData($entity, 'ItemGroup')) {
            $category = Category::DEFAULT_CATEGORY_NAME . self::DELIMITER_CATEGORY . $entity['ItemGroup'];
            if (isset($this->categories[$category])) {
                $conditions[] = [
                    'type' => Product::class,
                    'attribute' => 'category_ids',
                    'operator' => '==',
                    'value' => $this->categories[$category],
                    'is_value_processed' => 0
                ];
            }
        }
        if ($this->validateData($entity, 'Manufacturer')) {
            $conditions[] = [
                'type' => Product::class,
                'attribute' => 'manufacturer',
                'operator' => '==',
                'value' => $entity['Manufacturer'],
                'is_value_processed' => 0
            ];
        }
        if ($this->validateData($entity, 'ItemCode')) {
            $conditions[] = [
                'type' => Product::class,
                'attribute' => 'sku',
                'operator' => '==',
                'value' => $entity['ItemCode'],
                'is_value_processed' => 0
            ];
        }
        return $conditions;
    }

    /**
     * @return $this
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    protected function initCategories()
    {
        if (empty($this->categories)) {
            $stores = $this->storeManager->getStores();
            $searchStores = [\Magento\Store\Model\Store::DEFAULT_STORE_ID];
            foreach ($stores as $store) {
                $searchStores[] = $store->getId();
            }
            foreach ($searchStores as $store) {
                $collection = $this->categoryColFactory->create();
                $collection->setStoreId($store)
                    ->addAttributeToSelect(self::CATEGORY_COL_NAME);
                /** @var \Magento\Catalog\Model\Category $category */
                foreach ($collection as $category) {
                    $structure = explode(self::DELIMITER_CATEGORY, $category->getPath());
                    $pathSize = count($structure);
                    if ($pathSize > 1) {
                        $path = [];
                        for ($i = 1; $i < $pathSize; $i++) {
                            $path[] = $collection->getItemById((int)$structure[$i])->getName();
                        }
                        $index = implode(self::DELIMITER_CATEGORY, $path);
                        $this->categories[$index] = $category->getId();
                    } else {
                        $this->categories[$category->getName()] = $category->getId();
                    }
                }
            }
        }
        return $this;
    }

    /**
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    protected function initCustomerGroups()
    {
        if (empty($this->customerGroups)) {
            $magentoCustomerGroups = $this->groupRepository
                ->getList($this->searchCriteriaBuilder->create())->getItems();

            foreach ($magentoCustomerGroups as $magentoCustomerGroup) {
                $this->customerGroups[$magentoCustomerGroup->getCode()] = $magentoCustomerGroup->getId();
            }
        }
    }

    /**
     * @return string
     */
    protected function getAllCustomerGroups()
    {
        $customerGroups = '';
        foreach ($this->customerGroups as $customerGroupCode => $customerGroupId) {
            $customerGroups .= $customerGroupId . ',';
        }
        return rtrim($customerGroups, ",");
    }
}
