<?php
/**
 * Copyright © Firebear Studio, Inc. All rights reserved.
 * See COPYING.txt for license details.
 */
namespace Firebear\PlatformSap\Model\Import\Source;

use Magento\Catalog\Model\ResourceModel\Category\CollectionFactory;
use Magento\SalesRule\Model\RuleRepository;
use Magento\SalesRule\Model\Rule\Condition\Combine;
use Magento\SalesRule\Model\Rule\Condition\Product\Combine as ProductCombine;
use Magento\SalesRule\Model\Rule\Condition\Product;
use Magento\Customer\Api\GroupRepositoryInterface;
use Magento\Framework\Api\SearchCriteriaBuilder;
use Magento\Framework\Serialize\Serializer\Json;
use Firebear\PlatformSap\Model\Import\Source\Gateway\SalesRule as CartPriceGateway;
use Magento\Framework\App\RequestInterface;
use Magento\Store\Model\StoreManagerInterface;
use Magento\SalesRule\Model\ResourceModel\Rule\CollectionFactory as SalesRuleCollectionFactory;
use Magento\SalesRule\Model\Rule\Condition\Product\Found as ConditionProductFound;

/**
 * Sap cart price source
 */
class SalesRule extends CatalogRule
{
    /**
     * @var SalesRuleCollectionFactory
     */
    protected $salesRuleColFactory;

    /**
     * @var array
     */
    protected $cartRules;

    /**
     * @var RuleRepository
     */
    protected $salesRuleRepository;

    /**
     * SalesRule constructor.
     *
     * @param CartPriceGateway           $gateway
     * @param RequestInterface           $request
     * @param Json                       $jsonSerializer
     * @param StoreManagerInterface      $storeManager
     * @param CollectionFactory          $categoryColFactory
     * @param GroupRepositoryInterface   $groupRepository
     * @param SearchCriteriaBuilder      $searchCriteriaBuilder
     * @param SalesRuleCollectionFactory $salesRuleColFactory
     * @param RuleRepository             $salesRuleRepository
     * @param null                       $data
     *
     * @throws \Exception
     */
    public function __construct(
        CartPriceGateway $gateway,
        RequestInterface $request,
        Json $jsonSerializer,
        StoreManagerInterface $storeManager,
        CollectionFactory $categoryColFactory,
        GroupRepositoryInterface $groupRepository,
        SearchCriteriaBuilder $searchCriteriaBuilder,
        SalesRuleCollectionFactory $salesRuleColFactory,
        RuleRepository $salesRuleRepository,
        $data = null
    ) {
        $this->gateway = $gateway;
        $this->request = $request;
        $this->jsonSerializer = $jsonSerializer;
        $this->storeManager = $storeManager;
        $this->categoryColFactory = $categoryColFactory;
        $this->groupRepository = $groupRepository;
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
        $this->salesRuleColFactory = $salesRuleColFactory;
        $this->salesRuleRepository = $salesRuleRepository;
        $this->parseEntities($data);
    }

    /**
     * @param $data
     * @return array|bool
     * @throws \Exception
     */
    protected function parseEntities($data)
    {
        $cartRules = $this->getCartRules();
        $this->entities = AbstractSource::parseEntities($data);
        if (!empty($this->entities)) {
            foreach ($this->entities as $key => $entity) {
                $entityConditions = [];
                $entityConditions[] = $this->prepareConditions($entity);
                $ruleName = $this->getRuleName($entity);

                if (!empty($entityConditions)) {
                    $conditions = [
                        'type' => Combine::class,
                        'attribute' => 0,
                        'operator' => 0,
                        'value' => '1',
                        'is_value_processed' => 0,
                        'aggregator' => "all",
                        'conditions' => $entityConditions
                    ];

                    $actions = [
                        'type' => ProductCombine::class,
                        'attribute' => 0,
                        'operator' => 0,
                        'value' => '1',
                        'is_value_processed' => 0,
                        'aggregator' => "all",
                    ];

                    $this->initCustomerGroups();
                    if ($this->validateData($entity, 'CustomerGroup')
                        && isset($this->customerGroups[$entity['CustomerGroup']])) {
                        $customerGroupIds = $this->customerGroups[$entity['CustomerGroup']];
                    } elseif ($this->validateData($entity, 'CustomerGroup')
                        && !isset($this->customerGroups[$entity['CustomerGroup']])) {
                        continue;
                    } else {
                        $customerGroupIds = $this->getAllCustomerGroups();
                    }

                    $sapInternalId = $entity['AbsEntry'] . '-' . $entity['ObjType'] . '-' . $entity['ObjKey'];

                    /**
                     * Calculate maximal discount.
                     * @see https://docs.magento.com/user-guide/marketing/price-rules-cart-buy-x-get-y-free.html
                     */
                    $freeY = $entity['ForFree'];
                    $buyX = $entity['UpTo'];
                    $maxFreeItems = $entity['PayFor'];
                    $maxDiscountQty = ($buyX + $freeY) * ($maxFreeItems / $freeY);

                    $this->entities[$key] = [
                        'name' => $ruleName,
                        'code' => '',
                        'uses_per_coupon' => 0,
                        'description' => '',
                        'from_date' => ($this->validateData($entity, 'ValidForm')) ? $entity['ValidForm'] : '',
                        'to_date' => ($this->validateData($entity, 'ValidForm')) ? $entity['ValidTo'] : '',
                        'conditions_serialized' => $this->jsonSerializer->serialize($conditions),
                        'actions_serialized' => $this->jsonSerializer->serialize($actions),
                        'uses_per_customer' => 0,
                        'customer_group_ids' => $customerGroupIds,
                        'is_active' => 1,
                        'stop_rules_processing' => 0,
                        'sort_order' => 0,
                        'simple_action' => 'buy_x_get_y',
                        'discount_amount' => $freeY,
                        'discount_qty' => $maxDiscountQty,
                        'discount_step' => $buyX,
                        'simple_free_shipping' => 0,
                        'apply_to_shipping' => 0,
                        'times_used' => 0,
                        'is_rss' => 1,
                        'coupon_type' => 1,
                        'use_auto_generation' => 0,
                        'website_ids' => self::DEFAULT_WEBSITE_IDS,
                        'sap_id' => $sapInternalId
                    ];

                    if (isset($cartRules[$sapInternalId])) {
                        $this->entities[$key]['rule_id'] = $cartRules[$sapInternalId];
                    }
                }
            }
        }

        if (!empty($this->entities)) {
            $this->deleteExistingRules();
        }
        $this->_colNames = array_keys($this->entities[0] ?? []);
        return $this->entities;
    }

    /**
     * @param array $entity
     *
     * @return string
     */
    protected function getRuleName(array $entity)
    {
        $name = 'SAP';

        if (!empty($entity['Type'])) {
            if ($entity['Type'] == 'C' && $this->validateData($entity, 'CustomerGroup')) {
                $name .= ' - Customer Group: ' . $entity['CustomerGroup'];
            } else {
                $name .= ' - All Customer Groups';
            }
        }
        if ($this->validateData($entity, 'ItemGroup')) {
            $name .= ' - Category: ' . $entity['ItemGroup'];
        }
        if ($this->validateData($entity, 'Manufacturer')) {
            $name .= ' - Manufacturer: ' . $entity['Manufacturer'];
        }
        if ($this->validateData($entity, 'ItemCode')) {
            $name .= ' - Product: ' . $entity['ItemCode'];
        }

        return $name;
    }

    /**
     * Delete unused rules
     *
     * @return $this
     * @throws \Magento\Framework\Exception\CouldNotDeleteException
     */
    protected function deleteExistingRules()
    {
        $existingRules = $this->getCartRules();
        $rulesToImport = [];
        if (!empty($this->entities)) {
            foreach ($this->entities as $rule) {
                $rulesToImport[$rule['sap_id']] = 1;
            }
        }

        $rulesToDelete = array_diff_key($existingRules, $rulesToImport);

        foreach ($rulesToDelete as $rule) {
            if ($rule > 0) {
                $this->salesRuleRepository->deleteById($rule);
            }
        }

        return $this;
    }

    /**
     *  Init Magento catalog rules
     */
    protected function getCartRules()
    {
        if (empty($this->cartRules)) {
            $this->cartRules = [];
            $cartRules = $this->salesRuleColFactory->create();
            foreach ($cartRules as $cartRule) {
                $sapInternalId = $cartRule->getData('sap_id');
                if (!empty($sapInternalId)) {
                    $this->cartRules[$sapInternalId] = $cartRule->getId();
                }
            }
        }
        return $this->cartRules;
    }

    /**
     * @param $entity
     * @return array
     */
    protected function prepareConditions($entity)
    {
        $this->initCategories();
        $result = [];
        $conditions = [];
        if ($this->validateData($entity, 'ItemGroup')) {
            $category = Category::DEFAULT_CATEGORY_NAME . self::DELIMITER_CATEGORY . $entity['ItemGroup'];
            if (isset($this->categories[$category])) {
                $conditions[] = [
                    'type' => Product::class,
                    'attribute' => 'category_ids',
                    'operator' => '==',
                    'value' => $this->categories[$category],
                    'is_value_processed' => 0,
                    'attribute_scope' => ''
                ];
            }
        }
        if ($this->validateData($entity, 'Manufacturer')) {
            $conditions[] = [
                'type' => Product::class,
                'attribute' => 'manufacturer',
                'operator' => '==',
                'value' => $entity['Manufacturer'],
                'is_value_processed' => 0,
                'attribute_scope' => ''
            ];
        }
        if ($this->validateData($entity, 'ItemCode')) {
            $conditions[] = [
                'type' => Product::class,
                'attribute' => 'sku',
                'operator' => '==',
                'value' => $entity['ItemCode'],
                'is_value_processed' => 0,
                'attribute_scope' => ''
            ];
        }

        if (!empty($conditions)) {
            $result = [
                'type' => ConditionProductFound::class,
                'attribute' => 0,
                'operator' => 0,
                'value' => '1',
                'is_value_processed' => 0,
                'aggregator' => "all",
                'conditions' => $conditions
            ];
        }

        return $result;
    }
}
