<?php
/**
 * @copyright: Copyright © 2021 Firebear Studio. All rights reserved.
 * @author   : Firebear Studio <fbeardev@gmail.com>
 */

namespace Firebear\PlatformSap\Setup\Patch\Data;

use Magento\Eav\Model\Config as EavConfig;
use Magento\Eav\Model\Entity\Attribute\Set as AttributeSet;
use Magento\Eav\Setup\EavSetupFactory;
use Magento\Framework\Setup\ModuleDataSetupInterface;
use Magento\Framework\Setup\Patch\DataPatchInterface;
use Magento\Framework\Setup\Patch\PatchRevertableInterface;
use Magento\Eav\Model\Entity\Attribute\SetFactory as AttributeSetFactory;
use Magento\Customer\Api\AddressMetadataInterface;
use Magento\Customer\Model\Customer;


/**
 * Create custom customer attributes
 */
class UpdateCustomerAttributes implements DataPatchInterface, PatchRevertableInterface
{
    const ATTRIBUTE_CODE = 'sap_id';

    /**
     * @var ModuleDataSetupInterface
     */
    private $moduleDataSetup;

    /**
     * @var EavSetupFactory
     */
    private $eavSetupFactory;

    /**
     * @var EavSetup
     */
    private $setup;

    /**
     * @var EavConfig
     */
    private $eavConfig;

    /**
     * @var AttributeSetFactory
     */
    private $attributeSetFactory;

    /**
     * UpdateCustomerAttributes constructor.
     * @param ModuleDataSetupInterface $moduleDataSetup
     * @param EavSetupFactory $eavSetupFactory
     * @param EavConfig $eavConfig
     * @param AttributeSetFactory $attributeSetFactory
     */
    public function __construct(
        ModuleDataSetupInterface $moduleDataSetup,
        EavSetupFactory $eavSetupFactory,
        EavConfig $eavConfig,
        AttributeSetFactory $attributeSetFactory
    ) {
        $this->moduleDataSetup = $moduleDataSetup;
        $this->eavSetupFactory = $eavSetupFactory;
        $this->eavConfig = $eavConfig;
        $this->attributeSetFactory = $attributeSetFactory;
    }

    /**
     * {@inheritdoc}
     * @SuppressWarnings(PHPMD.ExcessiveMethodLength)
     */
    public function apply()
    {
       $this->addCustomerAttribute();
       $this->addCustomerAddressAttirbute();
    }

    /**
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    protected function addCustomerAttribute(){

        $setup = $this->getSetup();

        $customerEntity = $this->eavConfig->getEntityType('customer');
        $attributeSetId = $customerEntity->getDefaultAttributeSetId();

        /** @var $attributeSet AttributeSet */
        $attributeSet = $this->attributeSetFactory->create();
        $attributeGroupId = $attributeSet->getDefaultGroupId($attributeSetId);

        $setup->addAttribute(Customer::ENTITY, self::ATTRIBUTE_CODE, [
            'type' => 'varchar',
            'label' => __('Sap Id'),
            'input' => 'text',
            'required' => false,
            'visible' => true,
            'user_defined' => true,
            'position' =>999,
            'system' => 0,
        ]);

        $attribute = $this->eavConfig->getAttribute(
            Customer::ENTITY,
            self::ATTRIBUTE_CODE
        )->addData(
            [
                'attribute_set_id' => $attributeSetId,
                'attribute_group_id' => $attributeGroupId,
                'used_in_forms' => ['adminhtml_customer'],
            ]
        );

        $attribute->save();
    }

    /**
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    protected function addCustomerAddressAttirbute()
    {
        /** @var EavSetup $setup */
        $setup = $this->getSetup();

        $setup->addAttribute(
            AddressMetadataInterface::ENTITY_TYPE_ADDRESS,
            self::ATTRIBUTE_CODE,
            [
                'type' => 'varchar',
                'input' => 'text',
                'label' => __('Sap Id'),
                'visible' => true,
                'required' => false,
                'user_defined' => true,
                'system'=> false,
                'group'=> 'General',
                'global' => true,
                'visible_on_front' => false,
            ]
        );

        $customAttribute = $this->eavConfig->getAttribute(
            AddressMetadataInterface::ENTITY_TYPE_ADDRESS,
            self::ATTRIBUTE_CODE
        );

        $customAttribute->setData(
            'used_in_forms',
            ['adminhtml_customer_address']
        );
        $customAttribute->save();
    }

    /**
     * @return EavSetup
     */
    protected function getSetup()
    {
        if (!$this->setup) {
            $this->setup = $this->eavSetupFactory->create(['setup' => $this->moduleDataSetup]);
        }
        return $this->setup;
    }

    /**
     * @return array|string[]
     */
    public static function getDependencies()
    {
        return [];
    }

    /**
     * @return array|string[]
     */
    public function getAliases()
    {
        return [];
    }

    /**
     *  Remove custom customer attributes
     */
    public function revert()
    {
        $setup = $this->getSetup();
        $setup->removeAttribute(
            Customer::ENTITY,
            self::ATTRIBUTE_CODE
        );
        $setup->removeAttribute(
            AddressMetadataInterface::ENTITY_TYPE_ADDRESS,
            self::ATTRIBUTE_CODE
        );
    }
}
